@@ -9,12 +9,14 @@ import validictory
 
 class TestEnum(TestCase):
     schema = {"enum":["test", True, 123, ["???"]]}
+    schema2 = {"enum":("test", True, 123, ["???"])}
 
     def test_enum_pass(self):
         data = ["test", True, 123, ["???"]]
         try:
             for item in data:
                 validictory.validate(item, self.schema)
+                validictory.validate(item, self.schema2)
         except ValueError, e:
             self.fail("Unexpected failure: %s" % e)
 
@@ -51,12 +53,20 @@ class TestPattern(TestCase):
 
         self.assertRaises(ValueError, validictory.validate, data, self.schema)
 
+def validate_format_contains_spaces(validator, fieldname, value, format_option):
+    if ' ' in value:
+        return
+
+    raise validictory.ValidationError(
+        "Value %(value)r of field '%(fieldname)s' does not contain any spaces, but it should" % locals())
+
 class TestFormat(TestCase):
 
-    schema_datetime =    {"format": "date-time"}
-    schema_date =        {"format": "date"}
-    schema_time =        {"format": "time"}
+    schema_datetime    = {"format": "date-time"}
+    schema_date        = {"format": "date"}
+    schema_time        = {"format": "time"}
     schema_utcmillisec = {"format": "utc-millisec"}
+    schema_spaces      = {"format": "spaces"}
 
     def test_format_datetime_pass(self):
         data = "2011-01-13T10:56:53Z"
@@ -126,6 +136,46 @@ class TestFormat(TestCase):
         data = -1
         self.assertRaises(ValueError, validictory.validate, data, self.schema_utcmillisec)
 
+    def test_format_custom_unregistered_pass(self):
+        data = 'No-spaces-here'
+
+        try:
+            # no custom validator installed, so no error
+            validictory.validate(data, self.schema_spaces)
+        except ValueError, e:
+            self.fail("Unexpected failure: %s" % e)
+
+    def test_format_custom_instantiated_pass(self):
+        data = 'Here are spaces'
+
+        validator = validictory.SchemaValidator({'spaces': validate_format_contains_spaces})
+
+        try:
+            # validator installed, but data validates
+            validator.validate(data, self.schema_spaces)
+        except ValueError, e:
+            self.fail("Unexpected failure: %s" % e)
+
+    def test_format_custom_registered_pass(self):
+        data = 'Here are spaces'
+
+        validator = validictory.SchemaValidator()
+        validator.register_format_validator('spaces', validate_format_contains_spaces)
+
+        try:
+            # validator registered, but data validates
+            validator.validate(data, self.schema_spaces)
+        except ValueError, e:
+            self.fail("Unexpected failure: %s" % e)
+
+    def test_format_custom_registered_fail(self):
+        data = 'No-spaces-here'
+
+        validator = validictory.SchemaValidator({'spaces': validate_format_contains_spaces})
+
+        # validator registered, but data does not conform
+        self.assertRaises(ValueError, validator.validate, data, self.schema_spaces)
+
 
 class TestUniqueItems(TestCase):
 
@@ -202,7 +252,11 @@ class TestMaximum(TestCase):
         "prop01": { "type":"number", "maximum":10 },
         "prop02": { "type":"integer", "maximum":20 }
     }
+    props_exclusive = {
+        "prop": { "type":"integer", "maximum":20, "exclusiveMaximum": True },
+    }
     schema = {"type": "object", "properties":props}
+    schema_exclusive = {"type": "object", "properties": props_exclusive}
 
     def test_maximum_pass(self):
         #Test less than
@@ -216,6 +270,15 @@ class TestMaximum(TestCase):
         except ValueError, e:
             self.fail("Unexpected failure: %s" % e)
 
+    def test_maximum_exclusive_pass(self):
+        #Test less than
+        data = { "prop": 19 }
+
+        try:
+            validictory.validate(data, self.schema_exclusive)
+        except ValueError, e:
+            self.fail("Unexpected failure: %s" % e)
+
     def test_maximum_fail(self):
         #Test number
         data1 = { "prop01": 11, "prop02": 19 }
@@ -225,13 +288,23 @@ class TestMaximum(TestCase):
         self.assertRaises(ValueError, validictory.validate, data1, self.schema)
         self.assertRaises(ValueError, validictory.validate, data2, self.schema)
 
+    def test_maximum_exclusive_fail(self):
+        #Test equal
+        data = { "prop": 20 }
+
+        self.assertRaises(ValueError, validictory.validate, data, self.schema_exclusive)
+
 
 class TestMinimum(TestCase):
     props = {
         "prop01": { "type":"number", "minimum":10 },
         "prop02": { "type":"integer", "minimum":20 }
     }
+    props_exclusive = {
+        "prop": { "type":"integer", "minimum":20, "exclusiveMinimum": True },
+    }
     schema = {"type": "object", "properties":props}
+    schema_exclusive = {"type": "object", "properties": props_exclusive}
 
     def test_minimum_pass(self):
         #Test greater than
@@ -245,7 +318,16 @@ class TestMinimum(TestCase):
         except ValueError, e:
             self.fail("Unexpected failure: %s" % e)
 
-    def test_minumum_fail(self):
+    def test_minimum_exclusive_pass(self):
+        #Test greater than
+        data = { "prop": 21 }
+
+        try:
+            validictory.validate(data, self.schema_exclusive)
+        except ValueError, e:
+            self.fail("Unexpected failure: %s" % e)
+
+    def test_minimum_fail(self):
         #Test number
         data1 = { "prop01": 9, "prop02": 21 }
         #Test integer
@@ -254,6 +336,12 @@ class TestMinimum(TestCase):
         self.assertRaises(ValueError, validictory.validate, data1, self.schema)
         self.assertRaises(ValueError, validictory.validate, data2, self.schema)
 
+    def test_minimum_exclusive_fail(self):
+        #Test equal
+        data = { "prop": 20 }
+
+        self.assertRaises(ValueError, validictory.validate, data, self.schema_exclusive)
+
 
 class TestMinLength(TestCase):
     schema = { "minLength": 4 }
@@ -339,3 +427,24 @@ class TestBlank(TestCase):
             validictory.validate("test", {"blank":True})
         except ValueError, e:
             self.fail("Unexpected failure: %s" % e)
+
+
+class TestDivisibleBy(TestCase):
+    schema  = {'type': 'number', 'divisibleBy': 12}
+    schema0 = {'type': 'number', 'divisibleBy': 0}
+
+    def test_divisibleBy_pass(self):
+        data = 60
+        try:
+            validictory.validate(data, self.schema)
+        except ValueError, e:
+            self.fail("Unexpected failure: %s" % e)
+
+    def test_divisibleBy_fail(self):
+        data = 13
+        self.assertRaises(ValueError, validictory.validate, data, self.schema)
+
+    def test_divisibleBy_ZeroDivisionError_fail(self):
+        data = 60
+        self.assertRaises(ValueError, validictory.validate, data, self.schema0)
+
