@@ -1,6 +1,7 @@
 import re
 import copy
 from datetime import datetime
+import warnings
 
 class SchemaError(ValueError):
     """errors relating to an invalid schema passed to validate"""
@@ -8,10 +9,50 @@ class SchemaError(ValueError):
 class ValidationError(ValueError):
     """validation errors encountered during validate"""
 
+def _generate_datetime_validator(format_option, dateformat_string):
+    def validate_format_datetime(validator, fieldname, value, format_option):
+        try:
+            datetime.strptime(value, dateformat_string)
+        except ValueError, e:
+            raise ValidationError(
+                    "Value %(value)r of field '%(fieldname)s' is not in '%(format_option)s' format" % locals())
+
+    return validate_format_datetime
+
+validate_format_date_time  = _generate_datetime_validator('date-time', '%Y-%m-%dT%H:%M:%SZ')
+validate_format_date       = _generate_datetime_validator('date', '%Y-%m-%d')
+validate_format_time       = _generate_datetime_validator('time', '%H:%M:%S')
+
+def validate_format_utc_millisec(validator, fieldname, value, format_option):
+    if not isinstance(value, (int,float)):
+        raise ValidationError("Value %(value)r of field '%(fieldname)s' is not a number" % locals())
+
+    if not value > 0:
+        raise ValidationError("Value %(value)r of field '%(fieldname)s' is not a positive number" % locals())
+
+
+DEFAULT_FORMAT_VALIDATORS = {
+    'date-time'    : validate_format_date_time,
+    'date'         : validate_format_date,
+    'time'         : validate_format_time,
+    'utc-millisec' : validate_format_utc_millisec,
+}
+
 class SchemaValidator(object):
     '''
     Validator based on JSON Schema Proposal 2nd Draft.
     '''
+
+    def __init__(self, format_validators=None, required_by_default=True):
+        if format_validators is None:
+            format_validators = DEFAULT_FORMAT_VALIDATORS.copy()
+
+        self._format_validators = format_validators
+        self.required_by_default = required_by_default
+
+    def register_format_validator(self, format_name, format_validator_fun):
+        self._format_validators[format_name] = format_validator_fun
+
     def validate_type_string(self, val):
         return isinstance(val, basestring)
 
@@ -28,7 +69,7 @@ class SchemaValidator(object):
         return isinstance(val, dict)
 
     def validate_type_array(self, val):
-        return isinstance(val, list)
+        return isinstance(val, (list, tuple))
 
     def validate_type_null(self, val):
         return val is None
@@ -58,7 +99,7 @@ class SchemaValidator(object):
             value = x.get(fieldname)
 
         if fieldtype and fieldexists:
-            if isinstance(fieldtype, list):
+            if isinstance(fieldtype, (list, tuple)):
                 # Match if type matches any one of the types in the list
                 datavalid = False
                 for eachtype in fieldtype:
@@ -109,17 +150,17 @@ class SchemaValidator(object):
         '''
         if x.get(fieldname) is not None:
             value = x.get(fieldname)
-            if isinstance(value, list):
-                if isinstance(items, list):
-                    if len(items) == len(value):
+            if isinstance(value, (list, tuple)):
+                if isinstance(items, (list, tuple)):
+                    if not 'additionalItems' in schema.keys() and len(items) != len(value):
+                        self._error("Length of list %(value)r for field '%(fieldname)s' is not equal to length of schema list",
+                                    value, fieldname)
+                    else:
                         for itemIndex in range(len(items)):
                             try:
                                 self.validate(value[itemIndex], items[itemIndex])
                             except ValueError, e:
                                 raise type(e)("Failed to validate field '%s' list schema: %s" % (fieldname, e))
-                    else:
-                        self._error("Length of list %(value)r for field '%(fieldname)s' is not equal to length of schema list",
-                                    value, fieldname)
                 elif isinstance(items, dict):
                     for eachItem in value:
                         try:
@@ -129,12 +170,12 @@ class SchemaValidator(object):
                 else:
                     raise SchemaError("Properties definition of field '%s' is not a list or an object" % fieldname)
 
-    def validate_optional(self, x, fieldname, schema, optional=False):
+    def validate_required(self, x, fieldname, schema, required):
         '''
-        Validates that the given field is present if optional is false
+        Validates that the given field is present if required is True
         '''
         # Make sure the field is present
-        if fieldname not in x.keys() and not optional:
+        if fieldname not in x.keys() and required:
             self._error("Required field '%(fieldname)s' is missing",
                         None, fieldname)
 
@@ -147,6 +188,36 @@ class SchemaValidator(object):
             self._error("Value %(value)r for field '%(fieldname)s' cannot be blank'",
                         value, fieldname)
 
+    def validate_patternProperties(self, x, fieldname, schema, patternproperties=None):
+
+        if patternproperties == None:
+            patternproperties = {}
+
+        value_obj = x.get(fieldname)
+
+        for pattern, schema in patternproperties.iteritems():
+            for key, value in value_obj.iteritems():
+                if re.match(pattern, key):
+                    self.validate(value, schema)
+
+    def validate_additionalItems(self, x, fieldname, schema, additionalItems=False):
+        value = x.get(fieldname)
+
+        if not isinstance(value, (list, tuple)):
+            return
+
+        if isinstance(additionalItems, bool):
+            if additionalItems or 'items' not in schema.keys():
+                return
+            elif len(value) != len(schema['items']):
+                #print locals(), value, len(value), len(schema['items'])
+                self._error("Length of list %(value)r for field '%(fieldname)s' is not equal to length of schema list",
+                             value, fieldname)
+
+        remaining = value[len(schema['items']):]
+        if len(remaining) > 0:
+            self._validate(remaining, {'items': additionalItems})
+
     def validate_additionalProperties(self, x, fieldname, schema,
                                       additionalProperties=None):
         '''
@@ -179,20 +250,30 @@ class SchemaValidator(object):
             raise SchemaError("additionalProperties schema definition for field '%s' is not an object" % fieldname)
 
     def validate_requires(self, x, fieldname, schema, requires=None):
+        warnings.warn('The "requires" attribute has been replaced by "dependencies"', DeprecationWarning)
         if x.get(fieldname) is not None:
             if x.get(requires) is None:
                 self._error("Field '%(requires)s' is required by field '%(fieldname)s'",
                             None, fieldname, requires=requires)
 
+    def validate_dependencies(self, x, fieldname, schema, dependencies=None):
+        if x.get(fieldname) is not None:
+            if x.get(dependencies) is None:
+                self._error("Field '%(dependencies)s' is required by field '%(fieldname)s'",
+                            None, fieldname, dependencies=dependencies)
+
     def validate_minimum(self, x, fieldname, schema, minimum=None):
         '''
         Validates that the field is longer than or equal to the minimum
         length if specified
         '''
+
+        exclusive = schema.get('exclusiveMinimum', False)
+
         if x.get(fieldname) is not None:
             value = x.get(fieldname)
             if value is not None:
-                if type(value) in (int, float) and value < minimum:
+                if type(value) in (int, float) and (not exclusive and value < minimum) or (exclusive and value <= minimum):
                     self._error("Value %(value)r for field '%(fieldname)s' is less than minimum value: %(minimum)f",
                                 value, fieldname, minimum=minimum)
 
@@ -201,10 +282,13 @@ class SchemaValidator(object):
         Validates that the field is shorter than or equal to the maximum
         length if specified.
         '''
+
+        exclusive = schema.get('exclusiveMaximum', False)
+
         if x.get(fieldname) is not None:
             value = x.get(fieldname)
             if value is not None:
-                if type(value) in (int, float) and value > maximum:
+                if type(value) in (int, float) and (not exclusive and value > maximum) or (exclusive and value >= maximum):
                     self._error("Value %(value)r for field '%(fieldname)s' is greater than maximum value: %(maximum)f",
                                 value, fieldname, maximum=maximum)
 
@@ -215,7 +299,7 @@ class SchemaValidator(object):
         to the specified length
         '''
         value = x.get(fieldname)
-        if isinstance(value, (basestring, list)) and len(value) > length:
+        if isinstance(value, (basestring, list, tuple)) and len(value) > length:
             self._error("Length of value %(value)r for field '%(fieldname)s' must be less than or equal to %(length)d",
                         value, fieldname, length=length)
 
@@ -225,7 +309,7 @@ class SchemaValidator(object):
         to the specified length
         '''
         value = x.get(fieldname)
-        if isinstance(value, (basestring, list)) and len(value) < length:
+        if isinstance(value, (basestring, list, tuple)) and len(value) < length:
             self._error("Length of value %(value)r for field '%(fieldname)s' must be greater than or equal to %(length)d",
                         value, fieldname, length=length)
 
@@ -238,27 +322,10 @@ class SchemaValidator(object):
         '''
         value = x.get(fieldname)
 
-        date_formats = {
-            'date-time': '%Y-%m-%dT%H:%M:%SZ',
-            'date': '%Y-%m-%d',
-            'time': '%H:%M:%S',
-        }
+        format_validator = self._format_validators.get(format_option, None)
 
-        if isinstance(value, str) and format_option in date_formats.keys():
-            try:
-                datetime.strptime(value, date_formats[format_option])
-            except ValueError, e:
-                self._error("Value %(value)r of field '%(fieldname)s' is not in '%(format_option)s' format",
-                            value, fieldname, format_option=format_option)
-
-        if format_option == 'utc-millisec':
-            if not isinstance(value, (int,float)):
-                self._error("Value %(value)r of field '%(fieldname)s' is not a number",
-                            value, fieldname)
-
-            if not value > 0:
-                self._error("Value %(value)r of field '%(fieldname)s' is not a positive number",
-                            value, fieldname)
+        if format_validator:
+            format_validator(self, fieldname, value, format_option)
 
         # TODO: warn about unsupported format ?
 
@@ -274,14 +341,16 @@ class SchemaValidator(object):
                 self._error("Value %(value)r for field '%(fieldname)s' does not match regular expression '%(pattern)s'",
                             value, fieldname, pattern=pattern)
 
-    def validate_uniqueItems(self, x, fieldname, schema, unique=False):
+    def validate_uniqueItems(self, x, fieldname, schema, uniqueItems=False):
         '''
         Validates that all items in an array instance MUST be unique
         (contains no two identical values).
         '''
 
-        if not unique:
-            return # nothing to validate
+        # If additionalProperties is the boolean value True then we accept
+        # any additional properties.
+        if isinstance(uniqueItems, bool) and not uniqueItems:
+            return
 
         values = x.get(fieldname)
 
@@ -308,7 +377,7 @@ class SchemaValidator(object):
         '''
         value = x.get(fieldname)
         if value is not None:
-            if not isinstance(options, list):
+            if not isinstance(options, (list, tuple)):
                 raise SchemaError("Enumeration %r for field '%s' is not a list type", (options, fieldname))
             if value not in options:
                 self._error("Value %(value)r for field '%(fieldname)s' is not in the enumeration: %(options)r",
@@ -324,6 +393,19 @@ class SchemaValidator(object):
             raise SchemaError("The description for field '%s' must be a string."
                              % fieldname)
 
+    def validate_divisibleBy(self, x, fieldname, schema, divisibleBy=None):
+        value = x.get(fieldname)
+
+        if not self.validate_type_number(value):
+            return
+
+        if divisibleBy == 0:
+            raise SchemaError("'%r' <- divisibleBy can not be 0" % schema)
+
+        if value % divisibleBy != 0:
+            self._error("Value %(value)r field '%(fieldname)s' is not divisible by '%(divisibleBy)s'.",
+                        x.get(fieldname), fieldname, divisibleBy=divisibleBy)
+
     def validate_disallow(self, x, fieldname, schema, disallow=None):
         '''
         Validates that the value of the given field does not match the
@@ -353,8 +435,15 @@ class SchemaValidator(object):
 
             newschema = copy.copy(schema)
 
-            if 'optional' not in schema:
-                newschema['optional'] = False
+            # handle 'optional', replace it with 'required'
+            if 'required' in schema and 'optional' in schema:
+                raise SchemaError('cannot specify optional and required')
+            elif 'optional' in schema:
+                warnings.warn('The "optional" attribute has been replaced by "required"', DeprecationWarning)
+                newschema['required'] = not schema['optional']
+            elif 'required' not in schema:
+                newschema['required'] = self.required_by_default
+
             if 'blank' not in schema:
                 newschema['blank'] = False
 
@@ -369,4 +458,4 @@ class SchemaValidator(object):
 
         return data
 
-__all__ = ['JSONSchemaValidator']
+__all__ = ['SchemaValidator']
